@@ -338,7 +338,8 @@ static decode_state ff_decode(void) {
 
 		LOCK_O;
 		LOG_INFO("setting track_start");
-		output.next_sample_rate = decode_newstream(ff->codecC->sample_rate, output.max_sample_rate);
+		output.next_sample_rate = decode_newstream(ff->codecC->sample_rate, output.supported_rates);
+		IF_DSD(	output.next_dop = false; )
 		output.track_start = outputbuf->writep;
 		if (output.fade_mode) _checkfade(true);
 		decode.new_stream = false;
